<?php
namespace MINAV\Layersliderlight\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Stefan Grothus <info@made-in-nature.de>, made in nature
 *  Manfred Rutschmann <rutschmann@agentur-volumen.de>, VOLUMEN Werbe-/Internetagentur
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
use MINAV\Layersliderlight\Domain\Model\Slider;
use MINAV\Layersliderlight\Domain\Model\Slide;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Core\Resource\File;
use TYPO3\CMS\Core\Resource\FileReference;
use TYPO3\CMS\Core\Utility\File\BasicFileUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager;
use TYPO3\CMS\Extbase\Persistence\Generic\QueryResult;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;

/**
 *
 *
 * @package layersliderlight
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class SlideController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

    /**
     * slideRepository
     *
     * @var \MINAV\Layersliderlight\Domain\Repository\SlideRepository
     * @inject
     */
    protected $slideRepository;

    /**
     * @var \TYPO3\CMS\Extbase\Object\ObjectManager
     * @inject
     */
    protected $objectManager;

    /**
     * @var \TYPO3\CMS\Core\Resource\FileRepository
     * @inject
     */
    protected $fileRepository;


    /**
     * action list
     *
     * @return void
     */
    public function listAction() {
        /** @var QueryResult $slides */
        $slides = $this->slideRepository->findAll();
        $this->view->assign('slides', $slides);
    }

    /**
     * action show
     *
     * @param Slide $slide
     * @return void
     */
    public function showAction(Slide $slide) {
        $this->view->assign('slide', $slide);
    }

    /**
     * action new
     *
     * @param Slider $slider
     * @param Slide $newSlide
     * @dontvalidate $newSlide
     * @return void
     */
    public function newAction(Slider $slider, Slide $newSlide = NULL) {
        if($newSlide === NULL){
            /** @var Slide $newSlide */
            $newSlide = $this->objectManager->get('MINAV\\Layersliderlight\\Domain\\Model\\Slide');
            $newSlide->setTitle('New Slide');
            $newSlide->setSlider($slider);
            $newSlide->setSorting(99999);
        }
        $this->forward('create', NULL, NULL, array('newSlide' => $newSlide));
    }

    /**
     * @param Slider $slider
     * @param Slide $slide
     * @return void
     */
    public function pasteAction(Slider $slider, Slide $slide){

        /** @var Slide $slideCopy */
        $slideCopy = $this->objectManager->get('MINAV\\Layersliderlight\\Domain\\Model\\Slide');
        /** @var PersistenceManager $persistenceManager */
        $persistenceManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');

        $slideCopy->setTitle($slide->getTitle());
        $slideCopy->setConfiguration($slide->getConfiguration());
        $slideCopy->setSorting($slide->getSorting());
        $slideCopy->setSlider($slider);

        if($slide->getFalimage() !== NULL){
            /** @var FileReference $fileReference */
            $fileReference = $this->objectManager->get('MINAV\\Layersliderlight\\Domain\\Model\\FileReference');
            $fileReference->setFile($slide->getFalimage()->getOriginalResource()->getOriginalFile());
            $slideCopy->setFalimage($fileReference);
        }

        $slideCopy->setTitle('Copy from: '.$slideCopy->getTitle());
        $this->slideRepository->add($slideCopy);
        $persistenceManager->persistAll();
        $this->forward('edit', NULL, NULL, array('slide' => $slideCopy));
    }

    /**
     * action create
     *
     * @param Slide $newSlide
     * @return void
     */
    public function createAction(Slide $newSlide) {
        $this->slideRepository->add($newSlide);
        /** @var PersistenceManager $persistenceManager */
        $persistenceManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');
        $persistenceManager->persistAll();
        $this->forward('edit', NULL, NULL, array('slide' => $newSlide));
    }

    /**
     * action edit
     *
     * @param Slide $slide
     * @return void
     */
    public function editAction(Slide $slide) {
        $this->view->assign('fileWizardUrl', BackendUtility::getModuleUrl('wizard_element_browser',array('mode' => 'file', false, true)));
        $this->view->assign('linkWizardUrl', BackendUtility::getModuleUrl('wizard_element_browser',array('mode' => 'wizard', false, true)));
        $this->view->assign('slide', $slide);
        return $this->view->render();
    }

    /**
     * action update
     *
     * @param Slide $slide
     * @return void
     */
    public function updateAction(Slide $slide) {
        $this->slideRepository->update($slide);
        $this->addFlashMessage('Your Slide was updated.');
        $this->redirect('list');
    }

    /**
     * action delete
     *
     * @param Slide $slide
     * @return void
     */
    public function deleteAction(Slide $slide) {
        $into_table  ='sys_file_reference';
        $where_clause='uid_foreign ='.$slide->getUid().' AND tablenames="tx_layersliderlight_domain_model_slide" AND fieldname="falimage" AND table_local="sys_file"';
        $field_values = array('deleted' => 1);
        $GLOBALS['TYPO3_DB']->exec_UPDATEquery( $into_table, $where_clause, $field_values);

        $this->slideRepository->remove($slide);
    }

    /**
     * @param int $file
     * @param Slide $slide
     */
    public function linkFileAction($file, Slide $slide){
        /** @var File $falimage */
        $falimage = $this->fileRepository->findByUid($file);
        if($falimage instanceof File){

            $into_table  ='sys_file_reference';
            $where_clause='uid_foreign ='.$slide->getUid().' AND tablenames="tx_layersliderlight_domain_model_slide" AND fieldname="falimage" AND table_local="sys_file"';
            $field_values = array('deleted' => 1);

            $GLOBALS['TYPO3_DB']->exec_UPDATEquery( $into_table, $where_clause, $field_values);

            /** @var FileReference $fileReference */
            $fileReference = $this->objectManager->get('MINAV\\Layersliderlight\\Domain\\Model\\FileReference');
            $fileReference->setFile($falimage);
            $slide->setFalimage($fileReference);
            $this->slideRepository->update($slide);
            /** @var PersistenceManager $persistenceManager */
            $persistenceManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');
            $persistenceManager->persistAll();
            $this->addFlashMessage('New image attached to slide (autosave).', 'Success', AbstractMessage::OK);
        } else {
            $this->addFlashMessage('Error on select this File. Please try another image.', 'Error', AbstractMessage::ERROR);
        }
        $this->view->assign('slide', $slide);
        $this->view->assign('fileWizardUrl', BackendUtility::getModuleUrl('wizard_element_browser',array('mode' => 'file', false, true)));
        $this->view->assign('linkWizardUrl', BackendUtility::getModuleUrl('wizard_element_browser',array('mode' => 'wizard', false, true)));
    }

}
?>